// import 'x1b_defns.c' as x1b
#pragma once

/* Generated by xpdt
 * xpdt is written by Gianni Tedesco
 * https://www.scaramanga.co.uk
*/

#include <stdlib.h>

#include /*{x1b_hdr.path_spec}*/

/* ==== structs ==== */

/*#- for struct in namespace #*/

/*{x1b.struct_decl(struct)}*/
/*# if struct.needs_vbuf #*/

/*{x1b.serialized_struct_decl(struct)}*/

/*{x1b.blob_struct_decl(struct)}*/
/*# endif #*/
/*#- endfor -#*/

/*# for struct in namespace #*/


/* ==== /*{struct.name}*/ ==== */
/*{x1b.ctors(struct)}*/
/*{x1b.writers(struct)}*/
/*{x1b.readers(struct)}*/
/*#- endfor -#*/
